@@ -239,7 +239,7 @@ class VectorDict(defaultdict):
 
  **Direct Match**
 
->>> a_tree = dict( 
+ >>> a_tree = dict( 
             b = dict( 
                 c = 3.0, 
                 d = dict( e = True)
@@ -388,19 +388,20 @@ class VectorDict(defaultdict):
         """ Get a copy of an element at the coordinates given by the path
         Throw a KeyError excpetion if the path does not led to an element
  
+
  >>> intricated = convert_tree( { 'a' : { 'a' : { 'b' : { 'c' :  1 } } } } )
  >>> intricated.get_at( 'a', 'a', 'b' )
- >>> # defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'c': 1})
+ defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'c': 1})
  >>> intricated.get_at( 'a', 'a', 'b', 'c' )
- >>> # 1
+ 1
  >>> intricated.get_at( 'oops' )
- >>> # Traceback (most recent call last):
- >>> # File "<input>", line 1, in <module>
- >>> # File "vector_dict/VectorDict.py", line 304, in get_at
- >>> # return self.at( path, None , True)
- >>> # File "vector_dict/VectorDict.py", line 330, in at
- >>> # value = here[path[ -1 ] ]
- >>> # KeyError: 'oops'
+ Traceback (most recent call last):
+ File "<input>", line 1, in <module>
+ File "vector_dict/VectorDict.py", line 304, in get_at
+ return self.at( path, None , True)
+ File "vector_dict/VectorDict.py", line 330, in at
+ value = here[path[ -1 ] ]
+ KeyError: 'oops'
  
  """
         return self.at( path, None , True)
@@ -510,14 +511,6 @@ class VectorDict(defaultdict):
                
 
 
-    def __div__(left1, left2, *a, **lw):
-        """dict by dict division"""
-        if isinstance(left2, (float, int, complex)):
-            return left1.__rdiv__(left2)
-        if isinstance(left1, (float, int, complex)):
-            return left2.__rdiv__(left1)
-        else:
-            return left1.divide(left2)
    
     @__flatten_generator
     def find(self, predicate_on_path_value, path = tuple() ):
@@ -532,16 +525,46 @@ class VectorDict(defaultdict):
                 if predicate_on_path_value(Path( path + (k,) ), v):
                     yield  Element( path =  Path( path + (k,))  ,value = v  )
 
-                 
+    def  __rdiv__(self, other):
+        """reverse division"""
+        return self.__div__(other)
 
-    def __mul__(left1, left2, *a, **lw):
+    def __idiv__(self, other):
+        """immediate division /= """
+        return self.__opfactory__(other, False, lambda x,y: x/y , "__internal_divide__" )
+
+    def __div__(self, other):
+        """div with copy"""
+        return self.__opfactory__(other, True, lambda x,y: x/y , "__internal_divide__" )
+
+    def __rmul__(self, other):
+        return self.__mul__(other)
+
+    def __imul__(self, other):
+        return self.__opfactory__(other, False)
+
+    def __mul__(self, other):
+        return self.__opfactory__(other, True)
+
+    def __opfactory__(
+            self, 
+            other, 
+            copy = True, 
+            extern_operation = lambda x,y : x*y ,
+            intern_operation = "__internal_mul__"
+        ):
         """muler"""
-        if isinstance(left2, (float, int, complex)):
-            return left1.__rmul__(left2)
-        if isinstance(left1, (float, int, complex)):
-            return left2.__rmul__(left1)
+        ## __imul__ later
+        if not isinstance( self, VectorDict ):
+            self, other= other, self
+        a_copy = copy and self.copy() or self
+
+        if isinstance(other, (float, int, complex)):
+            for k, v in self.iteritems():
+                a_copy[k] = extern_operation( v , other)
+            return a_copy
         else:
-            return left1.homothetia(left2)
+            return getattr(a_copy, intern_operation)(other)
 
     def __neg__(self):
         for k, v in self.iteritems():
@@ -604,33 +627,153 @@ class VectorDict(defaultdict):
         arg["flatten"] = arg.get("flatten", True)
         return iter_object(self,(),**arg )
 
-    def divide(self, other):
-        """multiplying to vectors as one vector of homothetia * vector
-        it is a shortcut for a multiplication of a diagonal matrix
-        missing keys in the pseudo diagonal matrix are pruned"""
+    def intersection( self, other):
+        """Return all elements common in two different trees
+        raise an exception if both leaves are different
 
+ >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
+ >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 2  ) ) } )
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 1  ) ) } )
+ >>> a.intersection(b).tprint()
+ {
+     a = {
+         b = 1,
+     },
+ }
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1  ) ) } )
+ >>> a.intersection(b).tprint()
+ Traceback (most recent call last):
+   File "<input>", line 1, in <module>
+   File "vector_dict/VectorDict.py", line 634, in intersection
+     new_dict[k] = (self[k]).intersection( other[k] )
+   File "vector_dict/VectorDict.py", line 639, in intersection
+     other[k]
+ Exception: ('CollisionError', '2 != 1')
+ 
+
+"""
         common_key =  set( self.keys() ) &  set( other.keys() )
         new_dict = VectorDict(VectorDict, dict() )
         for k in common_key:
-            if  hasattr( self[k], "homothetia") :
-                new_dict[k] = (self[k]).divide( other[k] )
+            if  hasattr( self[k], "intersection") :
+                new_dict[k] = (self[k]).intersection( other[k] )
+            else:
+                if self[k] != other[k]:
+                    raise Exception("CollisionError","%s != %s" % (  
+                            self[k] ,
+                            other[k]
+                        ) )
+                new_dict[k] = self[k]
+        return new_dict
+
+    def union(self, other):
+        """return the union of two dicts
+
+ >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1  ) ) } )
+ >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
+ >>> b.union(a)
+ defaultdict(<type 'int'>, {'a': defaultdict(<type 'int'>, {'c': 1, 'b': 1, 'd': 2}), 'e': 1})
+ >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=3  ) ) } )
+ >>> b.union(a)
+ Traceback (most recent call last):
+   File "<input>", line 1, in <module>
+   File "vector_dict/VectorDict.py", line 669, in union
+     return self + other - self.intersection(other)
+   File "vector_dict/VectorDict.py", line 658, in intersection
+     new_dict[k] = (self[k]).intersection( other[k] )
+   File "vector_dict/VectorDict.py", line 663, in intersection
+     other[k]
+ Exception: ('CollisionError', '1 != 3')
+ 
+
+"""
+        return - self.intersection(other) + self + other
+    
+    def symmetric_difference( self, other):
+        """ return elements present only elements in one of the dict
+        Throw a Collision Error if two leaves in each tree are different
+
+ >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
+ >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1   ) ) } )
+ >>> a.symmetric_difference(b)
+ defaultdict(<type 'int'>, {'a': defaultdict(<type 'int'>, {'d': 2}), 'e': 1})
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 2   ) ) } )
+ >>> a.symmetric_difference(b)
+ Traceback (most recent call last):
+   File "<input>", line 1, in <module>
+   File "vector_dict/VectorDict.py", line 694, in symmetric_difference
+     for path,v in self.intersection(other).as_vector_iter():
+   File "vector_dict/VectorDict.py", line 658, in intersection
+     new_dict[k] = (self[k]).intersection( other[k] )
+   File "vector_dict/VectorDict.py", line 663, in intersection
+     other[k]
+ Exception: ('CollisionError', '1 != 2')
+
+
+"""
+        new_dict = self + other
+        for path,v in self.intersection(other).as_vector_iter():
+            new_dict.prune( *path )
+        return new_dict
+    def issubset( self, other ):
+        """tells if all element of self are included in other
+
+ >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
+ >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1   ) ) } )
+ >>> b.issubset(a)
+ True
+ >>> a.issubset(b)
+ False
+
+        """
+        return self.intersection(other) == self
+
+    def issuperset(self, other):
+        """tells if all element of other is included in self
+        throws an exception if two leaves in the two trees have different 
+        values
+ >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
+ >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
+ >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1   ) ) } )
+ >>> a.issuperset(b)
+ True
+ >>> b.issuperset(a)
+ False
+"""
+
+        return self.union(other) == self
+
+    def __internal_divide__(self, other):
+        """dividing two vectors internaly"""
+
+        common_key =  set( self.keys() ) &  set( other.keys() )
+        ## how could I get the factory of the default dict ? 
+        new_dict = VectorDict(VectorDict, dict() )
+        for k in common_key:
+            if  hasattr( self[k], "__internal_divide__") :
+                new_dict[k] = (self[k]).__internal_divide__( other[k] )
             else:
                 new_dict[k] = self[k] / other[k]
         return new_dict
 
-    def homothetia(self, other):
+    def __internal_mul__(self, other):
         """multiplying to vectors as one vector of homothetia * vector
         it is a shortcut for a multiplication of a diagonal matrix
         missing keys in the pseudo diagonal matrix are pruned"""
-
+        
         common_key =  set( self.keys() ) &  set( other.keys() )
+        ## how could I get the factory of the default dict ? 
         new_dict = VectorDict(VectorDict, dict() )
         for k in common_key:
-            if  hasattr( self[k], "homothetia") :
-                new_dict[k] = (self[k]).homothetia( other[k] )
+            if  hasattr( self[k], "__internal_mul__") :
+                new_dict[k] = (self[k]).__internal_mul__( other[k] )
             else:
                 new_dict[k] = self[k] * other[k]
         return new_dict
+    
     def dot(self, other):
         """scalar  = sum items self * other for each distinct key in common
         norm of the projection of self on other"""
@@ -660,15 +803,6 @@ class VectorDict(defaultdict):
 
             
 
-    def __imul__(integer, self):
-        if not isinstance(integer, int):
-            raise Exception("wrong type in dict multiplication")
-        #print "%r //%r" %  (self, integer)
-        another = self.copy()
-        #print "%r * %r " % (integer, another)
-        for k, v in self.iteritems():
-            another[k] = integer * v
-        return another
 
     def __sub__(self, other):
         """subber"""
@@ -683,27 +817,6 @@ class VectorDict(defaultdict):
                 positive[k] = -v
         return positive
 
-    def __rdiv__(self, scalar):
-        
-        #print "%r //%r" %  (self, integer)
-        another = self.copy()
-        if isinstance(scalar, VectorDict ):
-            return another.divide( scalar )
-        #print "%r * %r " % (integer, another)
-        for k, v in self.iteritems():
-            another[k] =  v / scalar
-        return another
-
-    def __rmul__(self, scalar):
-        #print "%r //%r" %  (self, integer)
-        another = self.copy()
-        if isinstance(scalar, VectorDict ):
-            return self.homothetia(scalar)
-            
-        #print "%r * %r " % (integer, another)
-        for k, v in self.iteritems():
-            another[k] = scalar * v
-        return another
 
     def __sub__(self, other):
         """subber"""
@@ -719,7 +832,7 @@ class VectorDict(defaultdict):
         return positive
 
     def tformat(self, indent_level = 0, base_indent = 4):
-        """pretty printing in a tree like form à la Perl"""
+        """pretty printing in a tree like form a la Perl"""
         offset = " " *  indent_level * base_indent
         toreturn = '{\n'
         for k,v in self.iteritems():
